#include <QWidget>
#include <QApplication>
#include <QDeclarativeView>
#include <QDeclarativeEngine>
#include <QGraphicsSceneMouseEvent>

const char *c_QmlMainFile = "qrc:qml/main.qml";

class GeometryFilter : public QObject {
public:
	GeometryFilter(QWidget *target):QObject(target), m_W(target) {}

protected:
	bool eventFilter(QObject *object, QEvent *event) {
		if (event->type() == QEvent::GraphicsSceneMousePress) {
			QGraphicsSceneMouseEvent *e =
				static_cast<QGraphicsSceneMouseEvent*>(event);
			m_GrabPos = e->scenePos().toPoint();
		}
		else if (event->type() == QEvent::GraphicsSceneMouseMove) {
			QGraphicsSceneMouseEvent *e =
				static_cast<QGraphicsSceneMouseEvent*>(event);
			if (e->buttons() & Qt::LeftButton)
				m_W->move(e->screenPos() - m_GrabPos);
		}
		return QObject::eventFilter(object, event);
	}

private:
	QWidget *m_W;
	QPoint m_GrabPos;
};


int main(int argc, char *argv[]) {
	QApplication a(argc, argv);

	QDeclarativeView v;
	v.scene()->installEventFilter(new GeometryFilter(&v));
	v.setWindowFlags(Qt::FramelessWindowHint);
	v.setAttribute(Qt::WA_TranslucentBackground);
	v.viewport()->setAutoFillBackground(false);
	v.setSource(QUrl(c_QmlMainFile));
	v.show();
	
	a.connect(v.engine(), SIGNAL(quit()), &a, SLOT(quit()));

	return a.exec();
}
